@@ -1,11 +1,7 @@
-import glob
 import json
 import logging
-import os
 import uuid
 from itertools import islice
-from tempfile import TemporaryDirectory
-
 import fiona
 import fiona.transform
 from django.contrib.auth.models import Group
@@ -22,24 +18,25 @@ except ImportError:  # TODO Remove when dropping Django releases < 3.1
     from django.contrib.postgres.fields import JSONField
 from django.contrib.gis.geos import WKBWriter
 from django.contrib.postgres.indexes import GistIndex, GinIndex
-from django.core.serializers import serialize
 from django.db import connection, transaction
 from django.db.models import Manager
 from django.db.models.signals import post_save
 from django.utils.functional import cached_property
 from django.utils.text import slugify
 from django.utils.translation import gettext_lazy as _
-from fiona.crs import from_epsg
 
-from . import GeometryTypes, settings as app_settings
+from . import GeometryTypes  # NOQA
+from . import settings as app_settings
 from .db.managers import FeatureQuerySet
 from .db.mixins import BaseUpdatableModel, LayerBasedModelMixin
-from .functions import ST_IsEmpty
-from .helpers import ChunkIterator, make_zipfile_bytesio
+from .functions import IsEmpty as IsEmptyFunc
+from .helpers import ChunkIterator
 from .routing.mixins import PgRoutingMixin, UpdateRoutingMixin
+
 from .signals import save_feature, save_layer_relation
+from .exports.helpers import generate_geojson, generate_shapefile
 from .tiles.decorators import zoom_update
-from .tiles.funcs import ST_HausdorffDistance
+from .tiles.funcs import HausdorffDistance
 from .validators import (validate_geom_type, validate_json_schema,
                          validate_json_schema_data)
 
@@ -169,69 +166,10 @@ class Layer(LayerBasedModelMixin, UpdateRoutingMixin):
             )
 
     def to_geojson(self):
-        return json.loads(serialize('geojson',
-                                    self.features.all(),
-                                    fields=('properties',),
-                                    geometry_field='geom',
-                                    properties_field='properties'))
+        return json.loads(generate_geojson(self))
 
     def to_shapefile(self):
-
-        if not self.features.count():
-            return
-
-        with TemporaryDirectory() as shape_folder:
-            shapes = {}
-            # get all accepted types if geom_type not defined, else keep selected
-            type_to_check = GeometryTypes.shape_allowed_type_names()\
-                if not self.geom_type else \
-                [self.geom_type.name]
-
-            # Create one shapefile by kind of geometry
-            for geom_type in type_to_check:
-                schema = {
-                    'geometry': geom_type,
-                    'properties': self.layer_properties,
-                }
-
-                shapes[geom_type] = fiona.open(
-                    shape_folder,
-                    layer=geom_type,
-                    mode='w',
-                    driver='ESRI Shapefile',
-                    schema=schema,
-                    encoding='UTF-8',
-                    crs=from_epsg(app_settings.INTERNAL_GEOMETRY_SRID)
-                )
-
-            # Export features to each kind of geometry
-            for feature in self.features.all():
-                shapes[feature.geom.geom_type].write({
-                    'geometry': json.loads(feature.geom.json),
-                    'properties': self._get_serialized_properties(feature.properties)
-                })
-
-            # Close fiona files
-            for geom_type, shape in shapes.items():
-                shape_size = len(shape)
-                shape.close()
-
-                # Delete empty shapes
-                if not shape_size:
-                    for filename in glob.iglob(os.path.join(shape_folder, f'{geom_type}.*')):
-                        os.remove(filename)
-
-            # Zip to BytesIO and return shape files
-            return make_zipfile_bytesio(shape_folder)
-
-    def _get_serialized_properties(self, feature_properties):
-        properties = {k: None for k in self.layer_properties}
-        for prop, value in feature_properties.items():
-            if isinstance(value, str):
-                properties[prop] = value
-            else:
-                properties[prop] = json.dumps(value)
-        return properties
+        return generate_shapefile(self)
 
     def _fiona_shape_projection(self, shape):
         ''' Return projection in EPSG format or raw Proj format extracted from
@@ -289,8 +227,8 @@ class Layer(LayerBasedModelMixin, UpdateRoutingMixin):
             nearest_feature = (
                 self.features
                     .filter(geom__bboverlaps=geometry)  # Bounding Box Overlap
-                    .annotate(hausdorff=ST_HausdorffDistance('geom',
-                                                             geometry.ewkb))
+                    .annotate(hausdorff=HausdorffDistance('geom',
+                                                          geometry))
                     .order_by('hausdorff')
                     .first()
             )
@@ -607,5 +545,5 @@ class FeatureExtraGeom(BaseUpdatableModel):
 
 
 @BaseSpatialField.register_lookup
-class IsEmpty(ST_IsEmpty):
+class IsEmpty(IsEmptyFunc):
     lookup_name = 'isempty'
